Attribute VB_Name = "MSTR"
'===============================================================================================
'This code is distributed "as is".
'No warranty of any kind is expressed or implied. You use it at your own risk.
'I disclaim all liabilities for consequential data loss, damages, loss of profits or any other
'kind of loss while using or misusing it.
'
'-----------------------------------------------------------------------------------------------
'Please ask for my approval at ssorin@gmail.com before distributing or posting
'this code to a site other than http://www.microstrategy101.com or http://microstrategyblog.com/
'===============================================================================================



'In this module we will have a look at three ways of finding an object
'1. FindObjectByName, where the name and type are specified
'2. FindObjectByID, where the ID and type are specified
'3. FindObjectbyID (overloaded function) where only the ID is specified

Function FindObjectByName(Name As String, DSSType As EnumDSSObjectType) As IDSSObjectInfo
        
        '==========================
        'This is the old school way
        '==========================

        
        Dim objsearch As IDSSSearch
        Dim result As IDSSFolder

        Set objsearch = objSession.ObjectSource.NewObject(DssTypeSearch)

        objsearch.NamePattern = Name
        objsearch.Flags = DssSearchNameWildCard
        objsearch.Types.Add (DSSType)


        Set result = objsearch.Execute

        'There may be more objects with the same name and type (though not recommended).
        'We shall keep the first found, if any.
        If result.Count > 0 Then
            Set FindObjectByName = result.Item(1)
        Else: Set FindObjectByName = Nothing
        End If





End Function

Function FindObjectByID2(ByVal ID As String, ByVal DSSType As EnumDSSObjectType) As IDSSObjectInfo
    
        '==============================
        'This is another old school way
        '==============================

        'FindObject is a native MicroStrategy function that searches an object based on its ID and type
        Set FindObjectByID2 = objSession.ObjectSource.FindObject(ID, DSSType)


End Function

Function FindObjectByID1(ByVal ID As String) As IDSSObjectInfo

        '======================
        'This is the real deal.
        '======================
        
        'Though slower, this function will eventually find any object, based on its ID alone.


        'The trick is to keep ignoring errors.
        On Error Resume Next
        'Otherwise, whenever FindObject attempts to search for the attribute with ID "xyz" but "xyz" is actually a metric you will get a show stopper error

        Dim result As IDSSObjectInfo
        

        'Here be some damn clever code
        'EnumDSSObjectType can also be expressed as a number. So far the greatest such number is 58
        For i = 1 To 58
            Set result = objSession.ObjectSource.FindObject(ID, i)
            
            'If an object is found, then stop processing the rest of the loop
            If Not result Is Nothing Then Exit For
        Next


        Set FindObjectByID1 = result


End Function

Public Function DSS2Name(numSubType As Integer) As String

    '======================================================================
    'This function returns the object type (in English) for a given SubType
    '======================================================================
    
    'While Type provides the general object type category (i.e. Report),
    'the detailed information can only be found in SubType (i.e. Grid, Graph, SQL)
    

Select Case numSubType
    Case 256
        DSS2Name = "Filter"
    Case 257
        DSS2Name = "Custom Group"
    Case 512
        DSS2Name = "Template"
    Case 768
        DSS2Name = "Grid"
    Case 769
        DSS2Name = "Graph"
    Case 770
        DSS2Name = "SQL"
    Case 774
        DSS2Name = "Grid Graph"
    Case 776
        DSS2Name = "Intelligent Cube"
    Case 1024
        DSS2Name = "Metric"
    Case 1025
        DSS2Name = "Subtotal"
    Case 1026
        DSS2Name = "Function"
    Case 1027
        DSS2Name = "Predictive Metric"
    Case 1536
        DSS2Name = "Autostyle"
    Case 1792
        DSS2Name = "Base Formula"
    Case 2048
        DSS2Name = "Folder"
    Case 2049
        DSS2Name = "Folder"
    Case 2304
        DSS2Name = "Device"
    Case 2563
        DSS2Name = "Text Prompt"
    Case 2564
        DSS2Name = "Numeric Prompt"
    Case 2565
        DSS2Name = "Date Prompt"
    Case 2566
        DSS2Name = "Object Prompt"
    Case 2567
        DSS2Name = "Elements Prompt"
    Case 2568
        DSS2Name = "Expression Prompt"
    Case 2816
        DSS2Name = "Function"
    Case 3072
        DSS2Name = "Attribute"
    Case 3074
        DSS2Name = "Transformation"
    Case 3328
        DSS2Name = "Fact"
    Case 3584
        DSS2Name = "Hierarchy"
    Case 3585
        DSS2Name = "Hierarchy"
    Case 3587
        DSS2Name = "Hierarchy"
    Case 3840
        DSS2Name = "Logical Table"
    Case 3842
        DSS2Name = "Warehouse Partition Mapping"
    Case 4608
        DSS2Name = "Shortcut"
    Case 5377
        DSS2Name = "Attribute Element"
    Case 5378
        DSS2Name = "Attribute Element"
    Case 5632
        DSS2Name = "Schema"
    Case 5888
        DSS2Name = "Schema"
    Case 6144
        DSS2Name = "Catalog"
    Case 6400
        DSS2Name = "Catalog Definition"
    Case 6656
        DSS2Name = "Column"
    Case 7168
        DSS2Name = "Property Set"
    Case 7424
        DSS2Name = "Database Instance"
    Case 7680
        DSS2Name = "Database Login"
    Case 7936
        DSS2Name = "Database Connection"
    Case 8192
        DSS2Name = "Project"
    Case 8448
        DSS2Name = "Project Source"
    Case 8704
        DSS2Name = "User"
    Case 8705
        DSS2Name = "User Group"
    Case 8960
        DSS2Name = "Transmitter"
    Case 9216
        DSS2Name = "Configuration"
    Case 9472
        DSS2Name = "Request"
    Case 9984
        DSS2Name = "Search"
    Case 10752
        DSS2Name = "Function Definition"
    Case 11009
        DSS2Name = "Transformation"
    Case 11264
        DSS2Name = "Security Role"
    Case 11520
        DSS2Name = "Language"
    Case 12032
        DSS2Name = "Consolidation"
    Case 12288
        DSS2Name = "Derived Element"
    Case 12544
        DSS2Name = "Event"
    Case 13056
        DSS2Name = "Schedule"
    Case 13312
        DSS2Name = "Link"
    Case 13568
        DSS2Name = "Logical Table"
    Case 13569
        DSS2Name = "Logical Table"
    Case 14080
        DSS2Name = "HTML Document"
    Case 14081
        DSS2Name = "Document"
    Case 14336
        DSS2Name = "Drill Map"
    Case 14592
        DSS2Name = "DBMS"
    Case 14848
        DSS2Name = "Security Filter"
    Case 15360
        DSS2Name = "Prompt Answer"
    Case 15616
        DSS2Name = "Graph Element"
    Case 772
        DSS2Name = "Data Mart Report"
    Case 775
        DSS2Name = "Non Interactive"
    
    'There are some SubTypes that aren't covered in this kit.
    'Feel free to improve this code with new findings.
    Case Else
        DSS2Name = Str(numSubType)
End Select


End Function
